<?php

namespace BexioConnectorByHelvy;

/**
 * Handle WooCommerce Webhooks Events
 *
 * @since 0.1.0
 */

defined('ABSPATH') || exit;

class WooCommerceWebhooks
{

	public function __construct()
	{
        add_action('woocommerce_new_order', array($this, 'handle_order_created'), 10, 2);
        add_action('woocommerce_subscription_payment_complete', array($this, 'handle_subscription_payment_complete'), 10, 2);
        add_action('woocommerce_subscription_renewal_payment_complete', array($this, 'handle_renewal_payment_complete'), 10, 2);
        add_action('woocommerce_order_status_processing', array($this, 'handle_order_paid'));
        add_action('woocommerce_order_status_completed', array($this, 'handle_order_completed'));
        add_filter('woocommerce_email_attachments', array($this, 'attach_invoice_pdf_to_wc_emails'), 10, 3 );
	}

    /**
     * Triggered when a new order is created
     * @param $order_id int Order ID
     * @param $order array Order
     * @return void
     */
	function handle_order_created($order_id, $order) {
        // Skip processing if the order is in a non-syncable state
        $order_status = $order->get_status();
        if (in_array($order_status, array('draft', 'cancelled', 'refunded', 'failed'))) {
            return;
        }

        WCToBexio::instance()->sync_order($order);
        $order = wc_get_order($order_id); // Re-fetch the order to get the updated meta
        WCToBexio::instance()->sync_order_payment($order);
        $order = wc_get_order($order_id); // Re-fetch the order to get the updated meta
        WCToBexio::instance()->sync_order_fulfillment($order);
    }

    /**
     * Handle subscription payment complete (which creates the first order of the subscription)
     * @param WC_Subscription $subscription Subscription object
     * @return void
     */
    public function handle_subscription_payment_complete($subscription) {
        $order = $subscription->get_last_order();
        if ($order) {
            $this->handle_order_created($order->get_id(), $order);
        }
    }

    /**
     * Handle subscription renewal payment complete (which creates the renewal order of the subscription)
     * @param WC_Subscription $subscription Subscription object
     * @param WC_Order $last_order Last order object
     * @return void
     */
    public function handle_renewal_payment_complete($subscription, $last_order) {
        $this->handle_order_created($last_order->get_id(), $last_order);
    }

    /**
     * Triggered when an order is paid
     * @param $order_id int Order ID
     * @return void
     */
	function handle_order_paid($order_id) {
        $order = wc_get_order($order_id);
        WCToBexio::instance()->sync_order_payment($order);
    }

    /**
     * Triggered when an order is completed
     * @param $order_id int Order ID
     * @return void
     */
	function handle_order_completed($order_id) {
        $order = wc_get_order($order_id);
        
        // Sync the order if it hasn't been synced yet
        if (!$order->get_meta(AdminAPISettings::$prefix . 'bexio_invoice_id')
            && !$order->get_meta(AdminAPISettings::$prefix . 'bexio_order_id')) {
            WCToBexio::instance()->sync_order($order);
            $order = wc_get_order($order_id); // Re-fetch the order to get the updated meta
            
            WCToBexio::instance()->sync_order_payment($order);
            $order = wc_get_order($order_id); // Re-fetch the order to get the updated meta
        }
        
        WCToBexio::instance()->sync_order_fulfillment($order);
    }

    /**
     * Attach the Bexio invoice PDF to WooCommerce emails
     * @param $attachments array Array of attachments
     * @param $email_id string Email ID
     * @param $order WC_Order Order object
     * @return array Array of attachments
     */
    function attach_invoice_pdf_to_wc_emails( $attachments, $email_id, $order ) {
        if (!$order instanceof \WC_Order) {
            return $attachments;
        }
        
        if (!get_option(AdminAPISettings::$prefix . 'attach_bexio_invoice_to_woocommerce_email')) {
            return $attachments;
        }
        
        // Check if this is the 'customer_processing_order' or 'customer_on_hold_order' email (email sent right after the order is placed)
        if ($email_id === 'customer_processing_order' || $email_id === 'customer_on_hold_order') {
            // Check for the invoice ID with a maximum of 10 attempts, one per second
            $max_attempts = 10;
            $attempts = 0;
            $invoice_id = '';
            while ($attempts < $max_attempts) {
                $invoice_id = $order->get_meta(AdminAPISettings::$prefix . 'bexio_invoice_id');
                if ($invoice_id) {
                    break;
                }
                $attempts++;
                sleep(1);
            }

            // Invoice not available
            if (!$invoice_id) {
                error_log("No invoice ID found after $max_attempts attempts for order " . $order->get_id());
                return $attachments;
            }

            $invoice_pdf = API::instance()->get_invoice_pdf($invoice_id);
            if ($invoice_pdf instanceof \WP_Error) {
                error_log("Not able to retrieve invoice PDF for order " . $order->get_id());
                return $attachments;
            }

            // Fetch the invoice in base64 format from an API
            $invoice_base64 = $invoice_pdf["content"];
            if ( $invoice_base64 ) {
                $decoded_pdf = base64_decode($invoice_base64);
                if ($decoded_pdf === false) {
                    error_log('Failed to decode base64 PDF invoice for order ID ' . $order->get_id());
                    return $attachments;
                }

                // Create a temporary file with a custom filename
                $upload_dir = wp_upload_dir();
                $temp_file_path = tempnam($upload_dir['basedir'], 'invoice-' . $order->get_id() . '-');
                $custom_temp_file_path = $temp_file_path . '.pdf';
                rename($temp_file_path, $custom_temp_file_path);

                $temp_file = fopen($custom_temp_file_path, 'w');
                $file_written = fwrite($temp_file, $decoded_pdf);
                if ($file_written === false) {
                    error_log('Failed to write the decoded PDF to the temporary file for order ID ' . $order->get_id());
                    return $attachments;
                }
                fclose($temp_file);

                // Attach the temporary file to the email with a custom filename
                add_action('phpmailer_init', function($phpmailer) use ($custom_temp_file_path, $order) {
                    $phpmailer->addAttachment($custom_temp_file_path, 'invoice-' . $order->get_id() . '.pdf');
                    register_shutdown_function(function() use ($custom_temp_file_path) {
                        unlink($custom_temp_file_path);
                    });
                });
            }
        }
    
        return $attachments;
    }
    
}
